@@ -236,6 +236,9 @@ void qtractorOptions::loadOptions (void)
 	bShiftKeyModifier = m_settings.value("/ShiftKeyModifier", false).toBool();
 	bMidButtonModifier = m_settings.value("/MidButtonModifier", false).toBool();
 	bMidiControlSync = m_settings.value("/MidiControlSync", false).toBool();
+	iExportRangeType = m_settings.value("/ExportRangeType", 0).toInt();
+	iExportRangeStart = (unsigned long) m_settings.value("/ExportRangeStart", 0).toUInt();
+	iExportRangeEnd = (unsigned long) m_settings.value("/ExportRangeEnd", 0).toUInt();
 	bExportAddTrack = m_settings.value("/ExportAddTrack", false).toBool();
 	m_settings.endGroup();
 
@@ -253,6 +256,7 @@ void qtractorOptions::loadOptions (void)
 	dssiPaths   = m_settings.value("/DssiPaths").toStringList();
 	vstPaths    = m_settings.value("/VstPaths").toStringList();
 	vst3Paths   = m_settings.value("/Vst3Paths").toStringList();
+	clapPaths   = m_settings.value("/ClapPaths").toStringList();
 	lv2Paths    = m_settings.value("/Lv2Paths").toStringList();
 	sLv2PresetDir = m_settings.value("/Lv2PresetDir").toString();
 	bAudioOutputBus = m_settings.value("/AudioOutputBus", false).toBool();
@@ -264,6 +268,7 @@ void qtractorOptions::loadOptions (void)
 	iDummyDssiHash = m_settings.value("/DummyDssiHash", 0).toInt();
 	iDummyVstHash = m_settings.value("/DummyVstHash", 0).toInt();
 	iDummyVst3Hash = m_settings.value("/DummyVst3Hash", 0).toInt();
+	iDummyClapHash = m_settings.value("/DummyClapHash", 0).toInt();
 	iDummyLv2Hash = m_settings.value("/DummyLv2Hash", 0).toInt();
 	bLv2DynManifest = false;//m_settings.value("/Lv2DynManifest", false).toBool();
 	bSaveCurve14bit = true;//m_settings.value("/SaveCurve14bit", false).toBool();
@@ -541,6 +546,9 @@ void qtractorOptions::saveOptions (void)
 	m_settings.setValue("/ShiftKeyModifier", bShiftKeyModifier);
 	m_settings.setValue("/MidButtonModifier", bMidButtonModifier);
 	m_settings.setValue("/MidiControlSync", bMidiControlSync);
+	m_settings.setValue("/ExportRangeType", iExportRangeType);
+	m_settings.setValue("/ExportRangeStart", uint(iExportRangeStart));
+	m_settings.setValue("/ExportRangeEnd", uint(iExportRangeEnd));
 	m_settings.setValue("/ExportAddTrack", bExportAddTrack);
 	m_settings.endGroup();
 
@@ -558,6 +566,7 @@ void qtractorOptions::saveOptions (void)
 	m_settings.setValue("/DssiPaths", dssiPaths);
 	m_settings.setValue("/VstPaths", vstPaths);
 	m_settings.setValue("/Vst3Paths", vst3Paths);
+	m_settings.setValue("/ClapPaths", vst3Paths);
 	m_settings.setValue("/Lv2Paths", lv2Paths);
 	m_settings.setValue("/Lv2PresetDir", sLv2PresetDir);
 	m_settings.setValue("/AudioOutputBus", bAudioOutputBus);
@@ -569,6 +578,7 @@ void qtractorOptions::saveOptions (void)
 	m_settings.setValue("/DummyDssiHash", iDummyDssiHash);
 	m_settings.setValue("/DummyVstHash", iDummyVstHash);
 	m_settings.setValue("/DummyVst3Hash", iDummyVst3Hash);
+	m_settings.setValue("/DummyClapHash", iDummyClapHash);
 	m_settings.setValue("/DummyLv2Hash", iDummyLv2Hash);
 	m_settings.setValue("/Lv2DynManifest", bLv2DynManifest);
 	m_settings.setValue("/SaveCurve14bit", bSaveCurve14bit);
