@@ -1,7 +1,7 @@
 // qtractorTrackTime.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2021, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2022, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -460,6 +460,7 @@ void qtractorTrackTime::mousePressEvent ( QMouseEvent *pMouseEvent )
 			if (bModifier) {
 				// Play-head positioning commit...
 				pTrackView->setPlayHead(iFrame);
+				pTrackView->setPlayHeadAutoBackward(iFrame);
 				pSession->setPlayHead(iFrame);
 			} else {
 				// Edit cursor (merge) positioning...
@@ -622,6 +623,7 @@ void qtractorTrackTime::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 		case DragPlayHead:
 			// Play-head positioning commit...
 			pTrackView->setPlayHead(iFrame);
+			pTrackView->setPlayHeadAutoBackward(iFrame);
 			pSession->setPlayHead(iFrame);
 			// Not quite a selection, rather just
 			// for immediate visual feedback...
@@ -689,7 +691,7 @@ void qtractorTrackTime::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 			if (bModifier) {
 				// Playhead positioning...
 				pTrackView->setPlayHead(iFrame);
-				// Immediately commited...
+				pTrackView->setPlayHeadAutoBackward(iFrame);
 				pSession->setPlayHead(iFrame);
 			} else {
 				// Deferred left-button edit-head positioning...
