@@ -52,8 +52,8 @@
 #include <QVBoxLayout>
 
 #include <QTimer>
-
 #include <QTimerEvent>
+
 #include <QResizeEvent>
 #include <QShowEvent>
 #include <QCloseEvent>
@@ -63,7 +63,6 @@
 
 #include <QRegularExpression>
 
-
 #if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 #define CONFIG_VST3_XCB
 #endif
@@ -72,6 +71,11 @@
 #include <xcb/xcb.h>
 #endif
 
+#if 0//QTRACTOR_VST3_EDITOR_TOOL
+#include "qtractorMainForm.h"
+#include "qtractorOptions.h"
+#endif
+
 
 using namespace Steinberg;
 using namespace Linux;
@@ -166,23 +170,28 @@ private:
 
 	unsigned int m_timerRefCount;
 
-	struct TimerItem
+	struct TimerHandlerItem
 	{
-		TimerItem(ITimerHandler *h, TimerInterval i)
+		TimerHandlerItem(ITimerHandler *h, TimerInterval i)
 			: handler(h), interval(i), counter(0) {}
 
+		void reset(TimerInterval i)
+			{ interval = i; counter = 0; }
+
 		ITimerHandler *handler;
 		TimerInterval  interval;
 		TimerInterval  counter;
 	};
 
-	QHash<ITimerHandler *, TimerItem *> m_timers;
-	QMultiHash<IEventHandler *, int> m_fileDescriptors;
+	QHash<ITimerHandler *, TimerHandlerItem *> m_timerHandlers;
+	QList<TimerHandlerItem *> m_timerHandlerItems;
+
+	QMultiHash<IEventHandler *, int> m_eventHandlers;
 
 #ifdef CONFIG_VST3_XCB
-	xcb_connection_t   *m_pXcbConnection;
-	int                 m_iXcbFileDescriptor;
-#endif	// defined(XCB_TEST)
+	xcb_connection_t *m_pXcbConnection;
+	int               m_iXcbFileDescriptor;
+#endif
 
 	Vst::ProcessContext m_processContext;
 	unsigned int        m_processRefCount;
@@ -655,7 +664,7 @@ tresult qtractorVst3PluginHost::registerEventHandler (
 #ifdef CONFIG_DEBUG
 	qDebug("qtractorVst3PluginHost::registerEventHandler(%p, %d)", handler, int(fd));
 #endif
-	m_fileDescriptors.insert(handler, int(fd));
+	m_eventHandlers.insert(handler, int(fd));
 	return kResultOk;
 }
 
@@ -665,7 +674,7 @@ tresult qtractorVst3PluginHost::unregisterEventHandler ( IEventHandler *handler
 #ifdef CONFIG_DEBUG
 	qDebug("qtractorVst3PluginHost::unregisterEventHandler(%p)", handler);
 #endif
-	m_fileDescriptors.remove(handler);
+	m_eventHandlers.remove(handler);
 	return kResultOk;
 }
 
@@ -676,7 +685,13 @@ tresult qtractorVst3PluginHost::registerTimer (
 #ifdef CONFIG_DEBUG
 	qDebug("qtractorVst3PluginHost::registerTimer(%p, %u)", handler, uint(msecs));
 #endif
-	m_timers.insert(handler, new TimerItem(handler, msecs));
+	TimerHandlerItem *timer_handler = m_timerHandlers.value(handler, nullptr);
+	if (timer_handler) {
+		timer_handler->reset(msecs);
+	} else {
+		timer_handler = new TimerHandlerItem(handler, msecs);
+		m_timerHandlers.insert(handler, timer_handler);
+	}
 	m_pTimer->start(int(msecs));
 	return kResultOk;
 }
@@ -687,8 +702,12 @@ tresult qtractorVst3PluginHost::unregisterTimer ( ITimerHandler *handler )
 #ifdef CONFIG_DEBUG
 	qDebug("qtractorVst3PluginHost::unregisterTimer(%p)", handler);
 #endif
-	m_timers.remove(handler);
-	if (m_timers.isEmpty())
+	TimerHandlerItem *timer_handler = m_timerHandlers.value(handler, nullptr);
+	if (timer_handler) {
+		m_timerHandlers.remove(handler);
+		m_timerHandlerItems.append(timer_handler);
+	}
+	if (m_timerHandlers.isEmpty())
 		m_pTimer->stop();
 	return kResultOk;
 }
@@ -698,11 +717,11 @@ tresult qtractorVst3PluginHost::unregisterTimer ( ITimerHandler *handler )
 //
 void qtractorVst3PluginHost::processTimers (void)
 {
-	foreach (TimerItem *timer, m_timers) {
-		timer->counter += timerInterval();
-		if (timer->counter >= timer->interval) {
-			timer->handler->onTimer();
-			timer->counter = 0;
+	foreach (TimerHandlerItem *timer_handler, m_timerHandlers) {
+		timer_handler->counter += timerInterval();
+		if (timer_handler->counter >= timer_handler->interval) {
+			timer_handler->handler->onTimer();
+			timer_handler->counter = 0;
 		}
 	}
 }
@@ -730,9 +749,9 @@ void qtractorVst3PluginHost::processEventHandlers (void)
 #endif
 
 	QMultiHash<IEventHandler *, int>::ConstIterator iter
-		= m_fileDescriptors.constBegin();
-	for ( ; iter != m_fileDescriptors.constEnd(); ++iter) {
-		foreach (int fd, m_fileDescriptors.values(iter.key())) {
+		= m_eventHandlers.constBegin();
+	for ( ; iter != m_eventHandlers.constEnd(); ++iter) {
+		foreach (int fd, m_eventHandlers.values(iter.key())) {
 			FD_SET(fd, &rfds);
 			FD_SET(fd, &wfds);
 			FD_SET(fd, &efds);
@@ -746,9 +765,9 @@ void qtractorVst3PluginHost::processEventHandlers (void)
 
 	const int result = ::select(nfds, &rfds, &wfds, nullptr, &timeout);
 	if (result > 0)	{
-		iter = m_fileDescriptors.constBegin();
-		for ( ; iter != m_fileDescriptors.constEnd(); ++iter) {
-			foreach (int fd, m_fileDescriptors.values(iter.key())) {
+		iter = m_eventHandlers.constBegin();
+		for ( ; iter != m_eventHandlers.constEnd(); ++iter) {
+			foreach (int fd, m_eventHandlers.values(iter.key())) {
 				if (FD_ISSET(fd, &rfds) ||
 					FD_ISSET(fd, &wfds) ||
 					FD_ISSET(fd, &efds)) {
@@ -857,10 +876,11 @@ void qtractorVst3PluginHost::clear (void)
 	m_timerRefCount = 0;
 	m_processRefCount = 0;
 
-	qDeleteAll(m_timers);
-	m_timers.clear();
+	qDeleteAll(m_timerHandlerItems);
+	m_timerHandlerItems.clear();
+	m_timerHandlers.clear();
 
-	m_fileDescriptors.clear();
+	m_eventHandlers.clear();
 
 	::memset(&m_processContext, 0, sizeof(Vst::ProcessContext));
 }
@@ -880,8 +900,8 @@ public:
 
 	// Constructor.
 	Impl (qtractorPluginFile *pFile) : m_pFile(pFile),
-			m_component(nullptr), m_controller(nullptr),
-			m_unitInfos(nullptr), m_iUniqueID(0) {}
+		m_component(nullptr), m_controller(nullptr),
+		m_unitInfos(nullptr), m_iUniqueID(0) {}
 
 	// Destructor.
 	~Impl () { close(); }
@@ -2392,7 +2412,8 @@ tresult qtractorVst3Plugin::Impl::notify ( Vst::IMessage *message )
 }
 
 
-// Audio processor stuff (TODO)...
+// Audio processor stuff...
+//
 bool qtractorVst3Plugin::Impl::process_reset (
 	qtractorAudioEngine *pAudioEngine )
 {
@@ -2955,7 +2976,7 @@ qtractorVst3Plugin::qtractorVst3Plugin (
 qtractorVst3Plugin::~qtractorVst3Plugin (void)
 {
 	// Cleanup all plugin instances...
-	setChannels(0);
+	cleanup();	// setChannels(0);
 
 	// Deallocate I/O audio buffer pointers.
 	if (m_ppIBuffer)
@@ -3144,6 +3165,8 @@ void qtractorVst3Plugin::updateParam (
 // All parameters update method.
 void qtractorVst3Plugin::updateParamValues ( bool bUpdate )
 {
+	int nupdate = 0;
+
 	// Make sure all cached parameter values are in sync
 	// with plugin parameter values; update cache otherwise.
 	const qtractorPlugin::Params& params = qtractorPlugin::params();
@@ -3156,10 +3179,13 @@ void qtractorVst3Plugin::updateParamValues ( bool bUpdate )
 			const float fValue = float(m_pImpl->getParameter(id));
 			if (pParam->value() != fValue) {
 				pParam->setValue(fValue, bUpdate);
-				updateFormDirtyCount();
+				++nupdate;
 			}
 		}
 	}
+
+	if (nupdate > 0)
+		updateFormDirtyCount();
 }
 
 
@@ -3205,13 +3231,15 @@ void qtractorVst3Plugin::freezeConfigs (void)
 	if (!type()->isConfigure())
 		return;
 
+	clearConfigs();
+
 	QByteArray data;
 
 	if (!m_pImpl->getState(data))
 		return;
 
 #ifdef CONFIG_DEBUG
-	qDebug("qtractorVstPlugin[%p]::freezeConfigs() chunk.size=%d", this, int(data.size()));
+	qDebug("qtractorVst3Plugin[%p]::freezeConfigs() data.size=%d", this, int(data.size()));
 #endif
 
 	// Set special plugin configuration item (base64 encoded)...
@@ -3278,7 +3306,20 @@ void qtractorVst3Plugin::openEditor ( QWidget *pParent )
 		return;
 	}
 
-	m_pEditorWidget = new EditorWidget(pParent, Qt::Window);
+	// What style do we create tool childs?
+	Qt::WindowFlags wflags = Qt::Window;
+#if 0//QTRACTOR_VST3_EDITOR_TOOL
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions && pOptions->bKeepToolsOnTop) {
+		wflags |= Qt::Tool;
+	//	wflags |= Qt::WindowStaysOnTopHint;
+		// Make sure it has a parent...
+		if (pParent == nullptr)
+			pParent = qtractorMainForm::getInstance();
+	}
+#endif
+
+	m_pEditorWidget = new EditorWidget(pParent, wflags);
 	m_pEditorWidget->setAttribute(Qt::WA_QuitOnClose, false);
 	m_pEditorWidget->setWindowTitle(pType->name());
 	m_pEditorWidget->setPlugin(this);
@@ -3485,7 +3526,7 @@ void qtractorVst3Plugin::process (
 						pMidiBuffer->push(&ev, event.sampleOffset);
 				}
 			}
-			pMidiManager->vst3_buffer_swap();
+			pMidiManager->swapOutputBuffers();
 		} else {
 			pMidiManager->resetOutputBuffers();
 		}
