@@ -1,7 +1,7 @@
 // qtractorVstPlugin.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2021, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2022, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -719,7 +719,7 @@ qtractorVstPlugin::qtractorVstPlugin (
 qtractorVstPlugin::~qtractorVstPlugin (void)
 {
 	// Cleanup all plugin instances...
-	setChannels(0);
+	cleanup();	// setChannels(0);
 
 	// Deallocate I/O audio buffer pointers.
 	if (m_ppIBuffer)
@@ -1073,6 +1073,8 @@ void qtractorVstPlugin::freezeConfigs (void)
 	if (!type()->isConfigure())
 		return;
 
+	clearConfigs();
+
 	AEffect *pVstEffect = vst_effect(0);
 	if (pVstEffect == nullptr)
 		return;
@@ -1350,6 +1352,8 @@ bool qtractorVstPlugin::x11EventFilter ( void *pvEvent )
 // All parameters update method.
 void qtractorVstPlugin::updateParamValues ( bool bUpdate )
 {
+	int nupdate = 0;
+
 	// Make sure all cached parameter values are in sync
 	// with plugin parameter values; update cache otherwise.
 	AEffect *pVstEffect = vst_effect(0);
@@ -1363,10 +1367,13 @@ void qtractorVstPlugin::updateParamValues ( bool bUpdate )
 				= pVstEffect->getParameter(pVstEffect, pParam->index());
 			if (pParam->value() != fValue) {
 				pParam->setValue(fValue, bUpdate);
-				updateFormDirtyCount();
+				++nupdate;
 			}
 		}
 	}
+
+	if (nupdate > 0)
+		updateFormDirtyCount();
 }
 
 
